@@ -13,3 +13,7 @@
 #undef GNOME_VFS_DEFAULT_BACKEND
 #undef USING_OAF
 #undef HAVE_GCONF
+#undef HAVE_OLDER_BZIP2
+#undef GNOME_PLATFORM_VERSION
+#undef ENABLE_PROFILER
+
