@@ -26,10 +26,11 @@
 
 
 #define _GNOME_VFS_METHOD_PARAM_CHECK(expression)			\
-	g_return_val_if_fail ((expression), GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail ((expression), GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 
-gboolean	gnome_vfs_method_init	(void);
-GnomeVFSMethod *gnome_vfs_method_get	(const gchar *name);
+gboolean	   gnome_vfs_method_init   (void);
+GnomeVFSMethod    *gnome_vfs_method_get    (const gchar *name);
+GnomeVFSTransform *gnome_vfs_transform_get (const gchar *name);
 
 #endif /* _GNOME_VFS_METHOD_H */
