@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /*
  * gnome-vfs-mime-sniff-buffer.c
  * Utility for implementing gnome_vfs_mime_type_from_magic, and other
@@ -88,6 +90,7 @@ gnome_vfs_mime_sniff_buffer_new_from_memory (const guchar *buffer,
 	result->buffer = g_malloc (buffer_length);
 	result->buffer_length = buffer_length;
 	memcpy (result->buffer, buffer, buffer_length);
+	result->read_whole_file = TRUE;
 
 	return result;
 }
@@ -124,49 +127,52 @@ gnome_vfs_mime_sniff_buffer_get (GnomeVFSMimeSniffBuffer *buffer,
 				 ssize_t size)
 {
 	GnomeVFSResult result;
-	GnomeVFSFileSize bytes_read;
+	GnomeVFSFileSize bytes_to_read, bytes_read;
 
+	/* check to see if we already have enough data */
 	if (buffer->buffer_length >= size) {
-		/* we already have enough data read */
 		return GNOME_VFS_OK;
 	}
 
-	if (!buffer->seek) {
+	/* if we've read the whole file, don't try to read any more */
+	if (buffer->read_whole_file) {
 		return GNOME_VFS_ERROR_EOF;
 	}
 
-	if (size < GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK) {
-		/* don't bother to read less than this */
-		size = GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK;
+	/* figure out how much to read */
+	bytes_to_read = size - buffer->buffer_length;
+
+	/* don't bother to read less than this */
+	if (bytes_to_read < GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK) {
+		bytes_to_read = GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK;
 	}
 
-	if (size - buffer->buffer_length < GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK) {
-		/* don't bother to add less than this */
-		size = buffer->buffer_length + GNOME_VFS_SNIFF_BUFFER_MIN_CHUNK;
-	}
-	
 	/* make room in buffer for new data */
-	buffer->buffer = g_realloc (buffer->buffer, size);
+	buffer->buffer = g_realloc (buffer->buffer,
+				    buffer->buffer_length + bytes_to_read);
 
-	/* seek to the spot we last took off */
-	result = (buffer->seek) (buffer->context, 
-				 GNOME_VFS_SEEK_START,
-				 buffer->buffer_length);
+	/* read in more data */
+	result = (* buffer->read) (buffer->context, 
+				   buffer->buffer + buffer->buffer_length,
+				   bytes_to_read,
+				   &bytes_read);
+	if (result == GNOME_VFS_ERROR_EOF) {
+		/* only happens with 0-byte files, due to other logic */
+		buffer->read_whole_file = TRUE;
+	}
 	if (result != GNOME_VFS_OK) {
 		return result;
 	}
+	if (bytes_read < bytes_to_read) {
+		buffer->read_whole_file = TRUE;
+	}
+	buffer->buffer_length += bytes_read;
 
-	/* read in more data */
-	result = (buffer->read) (buffer->context, 
-				 buffer->buffer + buffer->buffer_length,
-				 size - buffer->buffer_length,
-				 &bytes_read);
-
-	if (result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF) {
-		buffer->buffer_length += bytes_read;
-		result = GNOME_VFS_OK;
+	/* check to see if we have enough data */
+	if (buffer->buffer_length >= size) {
+		return GNOME_VFS_OK;
 	}
 
-	return result;
+	/* not enough data */
+	return GNOME_VFS_ERROR_EOF;
 }
-
