@@ -60,8 +60,8 @@ stop_after_log (const char *domain, GLogLevelFlags level,
 static void
 make_asserts_break (const char *domain)
 {
-	g_log_set_handler(domain, 
-		(GLogLevelFlags)(G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING),
+	g_log_set_handler (domain, 
+		(GLogLevelFlags) (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING),
 		stop_after_log, NULL);
 }
 
@@ -321,6 +321,8 @@ file_read_callback (GnomeVFSAsyncHandle *handle,
 	file_read_flag = TRUE;
 }
 
+static gboolean directory_load_flag;
+
 static void
 directory_load_callback (GnomeVFSAsyncHandle *handle,
 			 GnomeVFSResult result,
@@ -328,7 +330,66 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
 			 guint entries_read,
 			 gpointer callback_data)
 {
-	usleep(1000);
+	GnomeVFSFileInfo *element;
+
+	for (element = gnome_vfs_directory_list_first (list);
+		element != NULL;
+		element = gnome_vfs_directory_list_next (list)) {
+		gnome_vfs_file_info_ref (element);
+	}
+	
+	for (element = gnome_vfs_directory_list_first (list);
+		element != NULL;
+		element = gnome_vfs_directory_list_next (list)) {
+		gnome_vfs_file_info_unref (element);
+	}
+	
+	directory_load_flag = TRUE;
+}
+
+static gboolean directory_load_failed_flag;
+
+static void
+directory_load_failed_callback (GnomeVFSAsyncHandle *handle,
+				GnomeVFSResult result,
+				GnomeVFSDirectoryList *list,
+				guint entries_read,
+				gpointer callback_data)
+{
+	g_assert (result != GNOME_VFS_OK);
+	directory_load_failed_flag = TRUE;
+}
+
+static void
+test_open_read_close (void)
+{
+	file_open_flag = FALSE;
+	gnome_vfs_async_open (&test_handle,
+			      "file:///etc/passwd",
+			      GNOME_VFS_OPEN_READ,
+			      file_open_callback,
+			      test_callback_data);
+	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open: callback was not called"));
+
+	file_read_flag = FALSE;
+	gnome_vfs_async_read (test_handle,
+			      read_buffer,
+			      1,
+			      file_read_callback,
+			      test_callback_data);
+
+	TEST_ASSERT (wait_for_boolean (&file_read_flag), ("open read close: read callback was not called"));
+	file_closed_flag = FALSE;
+	gnome_vfs_async_close (test_handle,
+			       file_close_callback,
+			       test_callback_data);
+
+	TEST_ASSERT (wait_for_boolean (&file_closed_flag), ("open read close: close callback was not called"));
+
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open read cancel close: thread never went away"));
+	TEST_ASSERT (get_used_file_descriptor_count () == 0,
+		     ("open read cancel close: %d file descriptors leaked", get_used_file_descriptor_count ()));
+	free_at_start = get_free_file_descriptor_count ();
 }
 
 static void
@@ -364,6 +425,31 @@ test_open_read_cancel_close (void)
 	free_at_start = get_free_file_descriptor_count ();
 }
 
+static void
+test_open_close (void)
+{
+	file_open_flag = FALSE;
+	gnome_vfs_async_open (&test_handle,
+			      "file:///etc/passwd",
+			      GNOME_VFS_OPEN_READ,
+			      file_open_callback,
+			      test_callback_data);
+	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open: open callback was not called"));
+	
+	file_closed_flag = FALSE;
+	gnome_vfs_async_close (test_handle,
+			       file_close_callback,
+			       test_callback_data);
+
+
+	TEST_ASSERT (wait_for_boolean (&file_closed_flag), ("open close: close callback was not called"));
+	
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 1: thread never went away"));
+	TEST_ASSERT (get_used_file_descriptor_count () == 0,
+		     ("open cancel 1: %d file descriptors leaked", get_used_file_descriptor_count ()));
+	free_at_start = get_free_file_descriptor_count ();
+}
+
 static void
 test_open_cancel (void)
 {
@@ -427,17 +513,19 @@ test_open_fail (void)
 static void
 my_yield (int count)
 {
-	for (; count >= 0; count--) {
-		usleep (1000);
+	for (; count > 0; count--) {
+		usleep (100);
 		gtk_main_iteration_do (FALSE);
 	}
 }
 
 static void
-test_load_directory_cancel (int delay_till_cancel)
+test_load_directory_cancel (int delay_till_cancel, int chunk_count)
 {
 	GnomeVFSAsyncHandle *handle;
 	guint num_entries;
+	
+	
 	gnome_vfs_async_load_directory (&handle,
 					"file:///etc",
 					GNOME_VFS_FILE_INFO_GET_MIME_TYPE
@@ -448,13 +536,130 @@ test_load_directory_cancel (int delay_till_cancel)
 					GNOME_VFS_DIRECTORY_FILTER_NONE,
 					0,
 					NULL,
-					1,
+					chunk_count,
 					directory_load_callback,
 					&num_entries);
-	if (delay_till_cancel > 0)
-		my_yield (delay_till_cancel);
+	
+	usleep (delay_till_cancel * 100);
+	
+	directory_load_flag = FALSE;
 	gnome_vfs_async_cancel (handle);
 	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 1: thread never went away"));
+	TEST_ASSERT (!directory_load_flag, ("load directory cancel 1: load callback was called"));
+
+	gnome_vfs_async_load_directory (&handle,
+					"file:///etc",
+					GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+		 			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
+		 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
+					NULL,
+					FALSE,
+					GNOME_VFS_DIRECTORY_FILTER_NONE,
+					0,
+					NULL,
+					chunk_count,
+					directory_load_callback,
+					&num_entries);
+	
+	my_yield (delay_till_cancel);
+	
+	directory_load_flag = FALSE;
+	gnome_vfs_async_cancel (handle);
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 2: thread never went away"));
+	TEST_ASSERT (!directory_load_flag, ("load directory cancel 2: load callback was called"));
+}
+
+static void
+test_load_directory_fail (void)
+{
+	GnomeVFSAsyncHandle *handle;
+	guint num_entries;
+	
+	directory_load_failed_flag = FALSE;
+	gnome_vfs_async_load_directory (&handle,
+					"file:///strcprstskrzkrk",
+					GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+		 			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
+		 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
+					NULL,
+					FALSE,
+					GNOME_VFS_DIRECTORY_FILTER_NONE,
+					0,
+					NULL,
+					32,
+					directory_load_failed_callback,
+					&num_entries);
+		
+	TEST_ASSERT (wait_for_boolean (&directory_load_failed_flag), ("load directory cancel 1: load callback was not called"));
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 1: thread never went away"));
+}
+
+static gboolean find_directory_flag;
+
+static void
+test_find_directory_callback (GnomeVFSAsyncHandle *handle,
+			      GList *results,
+			      gpointer callback_data)
+{
+	GList *element;
+
+	find_directory_flag = TRUE;
+
+	for (element = results; element != NULL; element = element->next) {
+		GnomeVFSFindDirectoryResult *result_item = (GnomeVFSFindDirectoryResult *)element->data;
+		
+		if (result_item->result == GNOME_VFS_OK) {
+			gnome_vfs_uri_ref (result_item->uri);
+			gnome_vfs_uri_unref (result_item->uri);
+		}
+	}
+	
+	g_assert (callback_data == &find_directory_flag);
+}
+
+static void
+test_find_directory (int delay_till_cancel)
+{
+	GnomeVFSAsyncHandle *handle;
+	GList *vfs_uri_as_list;
+
+
+	vfs_uri_as_list = g_list_append (NULL, gnome_vfs_uri_new ("file://~"));
+	vfs_uri_as_list = g_list_append (vfs_uri_as_list, gnome_vfs_uri_new ("file:///ace_of_spades"));
+	
+	find_directory_flag = FALSE;
+	
+	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		test_find_directory_callback, &find_directory_flag);
+		
+	TEST_ASSERT (wait_for_boolean (&find_directory_flag), ("find directory cancel 1: callback was not called"));
+	
+	find_directory_flag = FALSE;
+	
+	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		test_find_directory_callback, &find_directory_flag);
+	
+	usleep (delay_till_cancel * 100);
+	
+	gnome_vfs_async_cancel (handle);
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 2: thread never went away"));
+	TEST_ASSERT (!find_directory_flag, ("find directory cancel 2: callback was called"));
+
+	
+	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		test_find_directory_callback, &find_directory_flag);
+	
+	my_yield (delay_till_cancel);
+	
+	find_directory_flag = FALSE;
+	gnome_vfs_async_cancel (handle);
+	TEST_ASSERT (wait_until_vfs_threads_gone (), ("open cancel 3: thread never went away"));
+	TEST_ASSERT (!find_directory_flag, ("find directory cancel 3: callback was called"));
+
+	gnome_vfs_uri_list_free (vfs_uri_as_list);
 }
 
 int
@@ -469,6 +674,7 @@ main (int argc, char **argv)
 	gnome_CORBA_init ("test-async-cancel", "0.0", &argc, argv, 0, &ev);
 	gnome_vfs_init ();
 
+	
 	/* Initialize our own stuff. */
 	free_at_start = get_free_file_descriptor_count ();
 
@@ -483,17 +689,46 @@ main (int argc, char **argv)
 	/* Test to see that a simple async. call works without leaking or anything. */
 	test_get_file_info ();
 	test_get_file_info ();
+	test_open_close ();
+	test_open_close ();
+	test_open_read_close ();
+	test_open_read_close ();
 	test_open_cancel ();
 	test_open_cancel ();
+
 	test_open_fail ();
 	test_open_fail ();
 	test_open_read_cancel_close ();
 	test_open_read_cancel_close ();
 
-	test_load_directory_cancel (0);
-	test_load_directory_cancel (1);
-	test_load_directory_cancel (10);
+	test_load_directory_fail ();
+	test_load_directory_cancel (0, 1);
+	test_load_directory_cancel (1, 1);
+	test_load_directory_cancel (10, 1);
+	test_load_directory_cancel (100, 1);
+	test_load_directory_cancel (0, 1);
+	test_load_directory_cancel (1, 1);
+	test_load_directory_cancel (10, 1);
+	test_load_directory_cancel (100, 1);
 
+	test_load_directory_cancel (0, 32);
+	test_load_directory_cancel (1, 32);
+	test_load_directory_cancel (10, 32);
+	test_load_directory_cancel (100, 32);
+	test_load_directory_cancel (0, 32);
+	test_load_directory_cancel (1, 32);
+	test_load_directory_cancel (10, 32);
+	test_load_directory_cancel (100, 32);
+
+	test_find_directory (0);
+	test_find_directory (0);
+	test_find_directory (1);
+	test_find_directory (1);
+	test_find_directory (10);
+	test_find_directory (10);
+	test_find_directory (100);
+	test_find_directory (100);
+		
 	gnome_vfs_shutdown ();
 	/* Report to "make check" on whether it all worked or not. */
 	return at_least_one_test_failed ? EXIT_FAILURE : EXIT_SUCCESS;
